import logging
import re

from checkov.common.models.consts import DOCKER_IMAGE_REGEX
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class ImagePullPolicyAlways(BaseK8Check):

    def __init__(self):
        """
        Image pull policy should be set to always to ensure you get the correct image and imagePullSecrets are correct
        Default is 'IfNotPresent' unless image tag is omitted or :latest
        https://kubernetes.io/docs/concepts/configuration/overview/#container-images

        An admission controller could be used to enforce imagePullPolicy
        """
        name = "Image Pull Policy should be Always"
        id = "CKV_K8S_15"
        # Location: container .imagePullPolicy
        supported_kind = ['containers', 'initContainers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return f'{conf["parent"]} - {conf["name"]}' if conf.get('name') else conf["parent"]

    def scan_spec_conf(self, conf):
        if "image" in conf:
            # Remove the digest, if present
            image_val = conf["image"]
            if not isinstance(image_val, str) or image_val.strip() == '':
                return CheckResult.UNKNOWN
            if '@' in image_val:
                image_val = image_val[0:image_val.index('@')]

            (image, tag) = re.findall(DOCKER_IMAGE_REGEX, image_val)[0]
            if "imagePullPolicy" not in conf:
                if tag == "latest" or tag == "":
                    # Default imagePullPolicy = Always
                    return CheckResult.PASSED
                else:
                    # Default imagePullPolicy = IfNotPresent
                    return CheckResult.FAILED
            else:
                if conf["imagePullPolicy"] != "Always":
                    return CheckResult.FAILED

        else:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = ImagePullPolicyAlways()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class LambdaEnvironmentEncryptionSettings(BaseResourceCheck):
    def __init__(self):
        name = "Check encryption settings for Lambda environmental variable"
        id = "CKV_AWS_173"
        supported_resources = ['AWS::Lambda::Function']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        properties = conf.get('Properties')
        if properties is not None:
            env = properties.get('Environment')
            if env is not None:
                if env.get('Variables') and not properties.get('KmsKeyArn'):
                    return CheckResult.FAILED
        return CheckResult.PASSED
		

check = LambdaEnvironmentEncryptionSettings()




from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class RootContainersHighUID(BaseK8Check):

    def __init__(self):
        name = "Containers should run as a high UID to avoid host conflict"
        # runAsUser should be >= 10000 at pod spec or container level
        # Location: Pod.spec.runAsUser
        # Location: CronJob.spec.jobTemplate.spec.template.spec.securityContext.runAsUser
        # Location: *.spec.template.spec.securityContext.runAsUser
        id = "CKV_K8S_40"
        supported_kind = ['Pod', 'Deployment', 'DaemonSet', 'StatefulSet', 'ReplicaSet', 'ReplicationController', 'Job', 'CronJob']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)



    def get_resource_id(self, conf):
        if "namespace" in conf["metadata"]:
            return "{}.{}.{}".format(conf["kind"], conf["metadata"]["name"], conf["metadata"]["namespace"])
        else:
            return "{}.{}.default".format(conf["kind"], conf["metadata"]["name"])

    def scan_spec_conf(self, conf):
        spec = {}

        if conf['kind'] == 'Pod':
            if "spec" in conf:
                spec = conf["spec"]
        elif conf['kind'] == 'CronJob':
            if "spec" in conf:
                if "jobTemplate" in conf["spec"]:
                    if "spec" in conf["spec"]["jobTemplate"]:
                        if "template" in conf["spec"]["jobTemplate"]["spec"]:
                            if "spec" in conf["spec"]["jobTemplate"]["spec"]["template"]:
                                spec = conf["spec"]["jobTemplate"]["spec"]["template"]["spec"]
        else:
            inner_spec = self.get_inner_entry(conf, "spec")
            spec = inner_spec if inner_spec else spec

        # Collect results
        if spec:
            results = {}
            results["pod"] = {}
            results["container"] = []
            results["pod"]["runAsUser"] = check_runAsUser(spec)

            if spec.get("containers"):
                for c in spec["containers"]:
                    cresults = {}
                    cresults["runAsUser"] = check_runAsUser(c)
                    results["container"].append(cresults)

            # Evaluate pass / fail - Container values override Pod values
                # Pod runAsUser >= 10000, no override at container (PASSED)
                # Pod runAsUser >= 10000, override at container < 10000 (FAILED)
                # Pod runAsUser < 10000, no override at container (FAILED)
                # Pod runAsUser < 10000, override at container >= 10000 (PASSED)
                # Pod runAsUser not set, container runAsUser not set or < 10000 (FAILED)
                # Pod runAsUser not set, container runAsUser set >= 10000 (PASSED)
            if results["pod"]["runAsUser"] == "PASSED":
                for cr in results["container"]:
                    if cr["runAsUser"] == "FAILED":
                        return CheckResult.FAILED
                return CheckResult.PASSED
            elif results["pod"]["runAsUser"] == "FAILED":
                containeroverride = False
                for cr in results["container"]:
                    if cr["runAsUser"] == "FAILED" or cr["runAsUser"] == "ABSENT":
                        return CheckResult.FAILED
                    elif cr["runAsUser"] == "PASSED":
                        containeroverride = True
                if containeroverride:
                    return CheckResult.PASSED
                return CheckResult.FAILED
            else:
                # Pod runAsUser ABSENT
                containeroverride = False
                for cr in results["container"]:
                    if cr["runAsUser"] == "FAILED" or cr["runAsUser"] == "ABSENT":
                        return CheckResult.FAILED
                    elif cr["runAsUser"] == "PASSED":
                        containeroverride = True
                if containeroverride:
                    return CheckResult.PASSED
                return CheckResult.FAILED

        return CheckResult.FAILED

check = RootContainersHighUID()

def check_runAsUser(spec):
    if spec.get("securityContext"):
        if "runAsUser" in spec["securityContext"]:
            if spec["securityContext"]["runAsUser"] >= 10000:
                return "PASSED"
            else:
                return "FAILED"
    return "ABSENT"





from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.kubernetes.base_spec_check import BaseK8Check
from checkov.common.util.type_forcers import force_list
import re


class NginxIngressCVE202125742AllSnippets(BaseK8Check):

    def __init__(self):
        name = "Prevent All NGINX Ingress annotation snippets. See CVE-2021-25742"
        id = "CKV_K8S_153"
        supported_kind = ['Ingress']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        if "namespace" in conf["metadata"]:
            return "{}.{}.{}".format(conf["kind"], conf["metadata"]["name"], conf["metadata"]["namespace"])
        else:
            return "{}.{}.default".format(conf["kind"], conf["metadata"]["name"])

    def scan_spec_conf(self, conf):

        if conf["metadata"]:
            if conf["metadata"].get('annotations'):
                for annotation in force_list(conf["metadata"]["annotations"]):
                    for key in annotation:
                        if "snippet" in key:
                            return CheckResult.FAILED
        return CheckResult.PASSED

check = NginxIngressCVE202125742AllSnippets()



from abc import abstractmethod
from typing import List, Callable, Optional, Dict, Any

from checkov.cloudformation.checks.resource.registry import cfn_registry
from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.multi_signature import multi_signature


class BaseResourceCheck(BaseCheck):
    def __init__(self, name: str, id: str, categories: List[CheckCategories], supported_resources: List[str],
                 guideline=None) -> None:
        super().__init__(
            name=name, id=id, categories=categories, supported_entities=supported_resources,
            block_type="resource", guideline=guideline
        )
        self.supported_resources = supported_resources
        cfn_registry.register(self)

    def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str) -> CheckResult:
        return self.scan_resource_conf(conf, entity_type)

    @multi_signature()
    @abstractmethod
    def scan_resource_conf(self, conf: Dict[str, Any], entity_type: str) -> CheckResult:
        raise NotImplementedError()

    @classmethod
    @scan_resource_conf.add_signature(args=["self", "conf"])
    def _scan_resource_conf_self_conf(cls, wrapped: Callable[..., CheckResult]) -> Callable[..., CheckResult]:
        def wrapper(self: BaseCheck, conf: Dict[str, Any], entity_type: Optional[str] = None) -> CheckResult:
            # keep default argument for entity_type so old code, that doesn't set it, will work.
            return wrapped(self, conf)

        return wrapper



import re

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

MULTI_STAGE_PATTERN = re.compile(r"(\S+)\s+as\s+(\S+)", re.IGNORECASE)


class ReferenceLatestTag(BaseDockerfileCheck):
    def __init__(self):
        name = "Ensure the base image uses a non latest version tag"
        id = "CKV_DOCKER_7"
        supported_instructions = ["FROM"]
        categories = [CheckCategories.CONVENTION]
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_entity_conf(self, conf):
        stages = []

        for content in conf:
            base_image = content["value"]
            multi_stage = re.match(MULTI_STAGE_PATTERN, base_image)
            if multi_stage:
                base_image = multi_stage[1]
                stages.append(multi_stage[2])

            if ":" not in base_image and base_image not in stages:
                return CheckResult.FAILED, content
            elif base_image.endswith(":latest"):
                return CheckResult.FAILED, content
        return CheckResult.PASSED, None


check = ReferenceLatestTag()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class FunctionAppsEnableAuthentication(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that function apps enables Authentication"
        id = "CKV_AZURE_56"
        supported_resources = ['azurerm_function_app']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'auth_settings/[0]/enabled'


check = FunctionAppsEnableAuthentication()



from typing import Generator, Any, Union


def get_inner_dict(source_dict, path_as_list):
    result = source_dict
    for index in path_as_list:
        result = result[index]
    return result


def merge_dicts(*dicts):
    """
    Merges two or more dicts. If there are duplicate keys, later dict arguments take precedence.

    Null, empty, or non-dict arguments are qiuetly skipped.
    :param dicts:
    :return:
    """
    res = {}
    for d in dicts:
        if not d or type(d) != dict:
            continue
        res = {**res, **d}
    return res


def generator_reader_wrapper(g: Generator) -> Union[None, Any]:
    try:
        return next(g)
    except StopIteration:
        return

def search_deep_keys(searchText, obj, path):
    """Search deep for keys and get their values"""
    keys = []
    if isinstance(obj, dict):
        for key in obj:
            pathprop = path[:]
            pathprop.append(key)
            if key == searchText:
                pathprop.append(obj[key])
                keys.append(pathprop)
                # pop the last element off for nesting of found elements for
                # dict and list checks
                pathprop = pathprop[:-1]
            if isinstance(obj[key], dict):
                if key != 'parent_metadata':
                    # Don't go back to the parent metadata, it is scanned for the parent
                    keys.extend(search_deep_keys(searchText, obj[key], pathprop))
            elif isinstance(obj[key], list):
                for index, item in enumerate(obj[key]):
                    pathproparr = pathprop[:]
                    pathproparr.append(index)
                    keys.extend(search_deep_keys(searchText, item, pathproparr))
    elif isinstance(obj, list):
        for index, item in enumerate(obj):
            pathprop = path[:]
            pathprop.append(index)
            keys.extend(search_deep_keys(searchText, item, pathprop))

    return keys


def find_in_dict(obj: dict, key_path: str) -> Any:
    val = obj
    key_list = key_path.split("/")
    for key in key_list:
        val = val.get(key)
        if val is None:
            return None
    return val


from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
import re
from typing import List

STO_NAME_REGEX = re.compile('^[a-z0-9]{3,24}$')


class StorageAccountName(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Storage Accounts adhere to the naming rules"
        id = "CKV_AZURE_43"
        supported_resources = ['azurerm_storage_account']
        categories = [CheckCategories.CONVENTION]
        super().__init__(name=name, id=id, categories=categories,
                         supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            The Storage Account naming reference:
            https://docs.microsoft.com/en-us/azure/storage/common/storage-account-overview#naming-storage-accounts
        :param conf: azurerm_storage_account configuration
        :return: <CheckResult>
        """
        return CheckResult.PASSED if conf.get('name') and re.findall(STO_NAME_REGEX, str(conf['name'][0])) else CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['name']


check = StorageAccountName()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class LivenessProbe(BaseK8Check):

    def __init__(self):
        name = "Liveness Probe Should be Configured"
        id = "CKV_K8S_8"
        # initContainers do not need Liveness Probes...
        # Location: container .livenessProbe
        supported_kind = ['containers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return f'{conf["parent"]} - {conf["name"]}' if conf.get('name') else conf["parent"]

    def scan_spec_conf(self, conf):
        # Don't check Job/CronJob
        if "parent" in conf:
            if "Job" in conf["parent"]:
                return CheckResult.PASSED
        if "livenessProbe" not in conf:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = LivenessProbe()



from checkov.common.parsers.node import DictNode
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class SecretManagerSecretEncrypted(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that Secrets Manager secret is encrypted using KMS CMK"
        id = "CKV_AWS_149"
        supported_resources = ["AWS::SecretsManager::Secret"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: DictNode) -> CheckResult:
        aws_kms_alias = "aws/"
        properties = conf.get("Properties")
        if properties:
            kms_key_id = properties.get("KmsKeyId")
            if kms_key_id and aws_kms_alias not in kms_key_id:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = SecretManagerSecretEncrypted()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck


class HealthcheckExists(BaseDockerfileCheck):
    def __init__(self):
        name = "Ensure that HEALTHCHECK instructions have been added to container images"
        id = "CKV_DOCKER_2"
        supported_instructions = ["*"]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_entity_conf(self, conf):
        for instruction, content in conf.items():
            if instruction == "HEALTHCHECK":
                return CheckResult.PASSED, conf[instruction][0]
        return CheckResult.FAILED, None


check = HealthcheckExists()



from abc import abstractmethod
from typing import Dict, Any

from checkov.common.checks.base_check import BaseCheck
from checkov.common.multi_signature import multi_signature
from checkov.kubernetes.registry import registry


class BaseK8Check(BaseCheck):
    def __init__(self, name, id, categories, supported_entities, guideline=None):
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities,
                         block_type="k8", guideline=guideline)
        self.supported_specs = supported_entities
        registry.register(self)

    @abstractmethod
    def get_resource_id(self, conf):
        pass

    def scan_entity_conf(self, conf, entity_type):
        return self.scan_spec_conf(conf, entity_type)

    @multi_signature()
    @abstractmethod
    def scan_spec_conf(self, conf, entity_type):
        raise NotImplementedError()

    @classmethod
    @scan_spec_conf.add_signature(args=["self", "conf"])
    def _scan_spec_conf_self_conf(cls, wrapped):
        def wrapper(self, conf, entity_type=None):
            # keep default argument for entity_type so old code, that doesn't set it, will work.
            return wrapped(self, conf)

        return wrapper

    @staticmethod
    def get_inner_entry(conf: Dict[str, Any], entry_name: str) -> Dict[str, Any]:
        spec = {}
        if conf.get("spec") and conf.get("spec").get("template"):
            spec = conf.get("spec").get("template").get(entry_name, {})
        return spec



import json
from abc import abstractmethod
from typing import Dict, List, Any

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.registry import resource_registry


class BaseResourceCheck(BaseCheck):
    def __init__(self, name: str, id: str, categories: List[CheckCategories], supported_resources: List[str],
                 guideline=None) -> None:
        super().__init__(
            name=name, id=id, categories=categories, supported_entities=supported_resources,
            block_type="resource", guideline=guideline
        )
        self.supported_resources = supported_resources
        resource_registry.register(self)

    def scan_entity_conf(self, conf: Dict[str, List[Any]], entity_type: str) -> CheckResult:
        if conf.get("count") == [0]:
            return CheckResult.UNKNOWN

        self.handle_dynamic_values(conf)
        return self.scan_resource_conf(conf)

    @abstractmethod
    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """
        self.evaluated_keys should be set with a JSONPath of the attribute inspected.
        If not relevant it should be set to an empty array so the previous check's value gets overridden in the report.
        """
        raise NotImplementedError()

    def handle_dynamic_values(self, conf: Dict[str, List[Any]]) -> None:
        # recursively search for blocks that are dynamic
        for block_name in conf.keys():
            if isinstance(conf[block_name], dict):
                self.handle_dynamic_values(conf[block_name])

            # if the configuration is a block element, search down again.
            if isinstance(conf[block_name], list) and len(conf[block_name]) > 0 and isinstance(conf[block_name][0], dict):
                self.handle_dynamic_values(conf[block_name][0])

        self.process_dynamic_values(conf)

    def process_dynamic_values(self, conf: Dict[str, List[Any]]) -> None:
        for dynamic_element in conf.get("dynamic", {}):
            if isinstance(dynamic_element, str):
                try:
                    dynamic_element = json.loads(dynamic_element)
                except Exception:
                    dynamic_element = {}

            for element_name in dynamic_element.keys():
                conf[element_name] = dynamic_element[element_name].get("content", [])



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DynamoDBTablesEncrypted(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DynamoDB Tables are encrypted using a KMS Customer Managed CMK"
        id = "CKV_AWS_119"
        supported_resources = ["aws_dynamodb_table"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if 'server_side_encryption' in conf.keys():
            sse = conf['server_side_encryption'][0]
            enabled = sse.get("enabled")
            kms_key_arn = sse.get("kms_key_arn")
            if enabled == [True] and kms_key_arn is not None:
                return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['server_side_encryption/[0]/enabled', 'server_side_encryption/[0]/kms_key_arn']


check = DynamoDBTablesEncrypted()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class KMSRotation(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure rotation for customer created CMKs is enabled"
        id = "CKV_AWS_7"
        supported_resources = ['aws_kms_key']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "enable_key_rotation"

    def scan_resource_conf(self, conf):
        # Only symmetric keys support auto rotation. The attribute is optional and defaults to symmetric.
        spec = conf.get('customer_master_key_spec')
        if not spec or 'SYMMETRIC_DEFAULT' in spec:
            return super().scan_resource_conf(conf)
        else:
            return CheckResult.PASSED


check = KMSRotation()



from checkov.arm.checks.resource.NSGRulePortAccessRestricted import NSGRulePortAccessRestricted


class NSGRuleRDPAccessRestricted(NSGRulePortAccessRestricted):
    def __init__(self):
        super().__init__(name="Ensure that RDP access is restricted from the internet", check_id="CKV_AZURE_9", port=3389)


check = NSGRuleRDPAccessRestricted()


from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class LambdaInVPC(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that AWS Lambda function is configured inside a VPC"
        id = "CKV_AWS_117"
        supported_resources = ['aws_lambda_function']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "vpc_config"

    def get_expected_value(self):
        return ANY_VALUE


check = LambdaInVPC()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import List


class ConfigConfigurationAggregator(BaseResourceCheck):
    def __init__(self):
        name = "Ensure AWS Config is enabled in all regions"
        id = "CKV_AWS_121"
        supported_resources = ['aws_config_configuration_aggregator']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for account_aggregation_source /  organization_aggregation_source
            at aws_config_configuration_aggregator:
            https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/config_configuration_aggregator#account-based-aggregation
        :param conf: aws_config_configuration_aggregator configuration
        :return: <CheckResult>
        """
        if "account_aggregation_source" in conf:
            aggregation_source = conf.get("account_aggregation_source", {})[0]
            if isinstance(aggregation_source, dict) and aggregation_source.get("all_regions"):
                return CheckResult.PASSED
        if "organization_aggregation_source" in conf:
            aggregation_source = conf.get("organization_aggregation_source", {})[0]
            if isinstance(aggregation_source, dict) and aggregation_source.get("all_regions"):
                return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ["account_aggregation_source", "organization_aggregation_source"]


check = ConfigConfigurationAggregator()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class GKEMetadataServerisEnabled(BaseResourceCheck):
    def __init__(self):
        name = "Ensure the GKE Metadata Server is Enabled"
        id = "CKV_GCP_69"
        supported_resources = ['google_container_cluster','google_container_node_pool']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)


    def scan_resource_conf(self, conf):
        """
        looks for workload_metadata_config in node_config which ensures that the metadata server is enabled
        https://registry.terraform.io/providers/hashicorp/google/latest/docs/resources/container_cluster
        :param conf: google_container_cluster or google_container_node_pool configuration
        :return: <CheckResult>
        """
        if 'node_config' in conf.keys():
            node = conf["node_config"][0]
            if isinstance(node, dict) and 'workload_metadata_config' in node:
                workload_metadata = node["workload_metadata_config"][0]
              
                if workload_metadata.get("mode", None) == ["GKE_METADATA"]:
                    return CheckResult.PASSED
                # deprecated in newer Google provider
                elif workload_metadata.get("node_metadata", None) == ["GKE_METADATA_SERVER"]:
                    return CheckResult.PASSED
            
            return CheckResult.FAILED
        return CheckResult.FAILED

check = GKEMetadataServerisEnabled()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GKEEnsureIntegrityMonitoring(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Integrity Monitoring for Shielded GKE Nodes is Enabled"
        check_id = "CKV_GCP_72"
        supported_resources = ['google_container_cluster', 'google_container_node_pool']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'node_config' in conf.keys():
            node = conf["node_config"][0]
            self.evaluated_keys = ['node_config']
            if isinstance(node, dict) and 'shielded_instance_config' in node.keys():
                monitor = node["shielded_instance_config"][0]
                if monitor.get("enable_integrity_monitoring") == [True]:
                    self.evaluated_keys = ['node_config/[0]/shielded_instance_config/[0]/enable_integrity_monitoring']
                    return CheckResult.PASSED
                self.evaluated_keys = ['node_config/[0]/shielded_instance_config']
                return CheckResult.FAILED
            # as default is true this is a pass
            return CheckResult.PASSED
        # no config is valid it could be in the the node_pool
        return CheckResult.UNKNOWN


check = GKEEnsureIntegrityMonitoring()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check

class ApiServerSecurityContextDenyPlugin(BaseK8Check):
    def __init__(self):
        id = "CKV_K8S_81"
        name = "Ensure that the admission control plugin SecurityContextDeny is set if PodSecurityPolicy is not used"
        categories = [CheckCategories.KUBERNETES]
        supported_entities = ['containers']
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities)

    def get_resource_id(self, conf):
        return f'{conf["parent"]} - {conf["name"]}' if conf.get('name') else conf["parent"]

    def scan_spec_conf(self, conf):
        if "command" in conf:
            if "kube-apiserver" in conf["command"]:
                for cmd in conf["command"]:
                    if cmd == "--enable-admission-plugins":
                        return CheckResult.FAILED  
                    if "=" in cmd:
                        [field,value,*_] = cmd.split("=")
                        if field == "--enable-admission-plugins":
                            if "PodSecurityPolicy" not in value and "SecurityContextDeny" not in value:
                                return CheckResult.FAILED 
                                                     
        return CheckResult.PASSED

check = ApiServerSecurityContextDenyPlugin()



from checkov.common.models.enums import CheckCategories
from checkov.kubernetes.base_spec_omitted_or_value_check import BaseSpecOmittedOrValueCheck


class ShareHostPIDPSP(BaseSpecOmittedOrValueCheck):

    def __init__(self):
        # CIS-1.3 1.7.2
        # CIS-1.5 5.2.2
        name = "Do not admit containers wishing to share the host process ID namespace"
        id = "CKV_K8S_1"
        # Location: PodSecurityPolicy.spec.hostPID
        supported_kind = ['PodSecurityPolicy']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_inspected_key(self):
        return "spec/hostPID"

    def get_resource_id(self, conf):
        if "metadata" in conf:
            if "name" in conf["metadata"]:
                return 'PodSecurityPolicy.{}'.format(conf["metadata"]["name"])
        return 'PodSecurityPolicy.spec.hostPID'


check = ShareHostPIDPSP()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class UsernameExists(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure email is set"
        id = "CKV_LIN_4"
        supported_resources = ["linode_user"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if conf.get("username"):
            return CheckResult.PASSED
        return CheckResult.FAILED


check = UsernameExists()



from typing import Dict, List, Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class QLDBLedgerDeletionProtection(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure QLDB ledger has deletion protection enabled"
        id = "CKV_AWS_172"
        supported_resources = ["aws_qldb_ledger"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "deletion_protection"


check = QLDBLedgerDeletionProtection()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CloudStorageSelfLogging(BaseResourceCheck):
    def __init__(self):
        name = "Bucket should not log to itself"
        id = "CKV_GCP_63"
        supported_resources = ['google_storage_bucket']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        bucket_name = conf['name']
        #check for logging
        if 'logging' in conf:
            self.evaluated_keys = ['logging']
            if conf['logging'][0]:
                log_bucket_name = conf['logging'][0]['log_bucket']
                self.evaluated_keys = ['logging/[0]/log_bucket', 'name']
                if log_bucket_name != bucket_name:
                    return CheckResult.PASSED
                return CheckResult.FAILED
            return CheckResult.FAILED
        return CheckResult.UNKNOWN

check = CloudStorageSelfLogging()



import json

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import is_json
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import List


class SQSPolicy(BaseResourceCheck):
    def __init__(self):
        name = "Ensure SQS policy does not allow ALL (*) actions."
        id = "CKV_AWS_72"
        supported_resources = ['aws_sqs_queue_policy']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for public * policy for SQS repository:
            https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/sqs_queue_policy
        :param conf: aws_sqs_queue_policy configuration
        :return: <CheckResult>
        """
        if "policy" in conf.keys() and is_json(conf["policy"][0]) \
                and json.loads(conf["policy"][0])['Statement'][0]['Action'] == '*':
            return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['policy']


check = SQSPolicy()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck

# https://docs.microsoft.com/en-us/azure/templates/microsoft.web/2019-08-01/sites


class AppServiceMinTLSVersion(BaseResourceCheck):
    def __init__(self):
        name = "Ensure web app is using the latest version of TLS encryption"
        id = "CKV_AZURE_15"
        supported_resources = ["Microsoft.Web/sites"]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        properties = conf.get("properties")
        if properties:
            site_config = properties.get("siteConfig")
            if site_config:
                min_tls_version = site_config.get("minTlsVersion")
                if min_tls_version == '1.2':
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = AppServiceMinTLSVersion()



import re

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import extract_policy_dict
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import List


class IAMRoleAllowAssumeFromAccount(BaseResourceCheck):

    def __init__(self):
        name = "Ensure IAM role allows only specific principals in account to assume it"
        id = "CKV_AWS_61"
        supported_resources = ['aws_iam_role']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if not conf.get('assume_role_policy') or not isinstance(conf['assume_role_policy'][0], str):
            return CheckResult.PASSED
        try:
            assume_role_block = extract_policy_dict(conf['assume_role_policy'][0])
            if assume_role_block and 'Statement' in assume_role_block.keys() \
                    and 'Principal' in assume_role_block['Statement'][0] \
                    and 'AWS' in assume_role_block['Statement'][0]['Principal']:
                account_access = re.compile(r'\d{12}|arn:aws:iam::\d{12}:root')
                if re.match(account_access, assume_role_block['Statement'][0]['Principal']['AWS']):
                    return CheckResult.FAILED
        except:  # nosec
            pass
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['assume_role_policy']


check = IAMRoleAllowAssumeFromAccount()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.secrets import string_has_secrets
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class VMCredsInCustomData(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that no sensitive credentials are exposed in VM custom_data"
        id = "CKV_AZURE_45"
        supported_resources = ['azurerm_virtual_machine']
        categories = [CheckCategories.SECRETS]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        os_profile = conf.get('os_profile')
        if os_profile:
            self.evaluated_keys = ['os_profile']
            os_profile = os_profile[0]
            custom_data = os_profile.get('custom_data')
            if custom_data:
                custom_data = custom_data[0]
                self.evaluated_keys = ['os_profile/[0]/custom_data']
                if isinstance(custom_data, str):
                    if string_has_secrets(custom_data):
                        return CheckResult.FAILED
        return CheckResult.PASSED


check = VMCredsInCustomData()


from checkov.common.util.config_utils import should_scan_hcl_files

SCAN_HCL_FLAG = "CKV_SCAN_HCL"
SUPPORTED_FILE_EXTENSIONS = [".tf", ".yml", ".yaml", ".json", ".template"]
if should_scan_hcl_files():
    SUPPORTED_FILE_EXTENSIONS.append(".hcl")
ANY_VALUE = "CKV_ANY"
DOCKER_IMAGE_REGEX = r'(?:[^\s\/]+/)?([^\s:]+):?([^\s]*)'
access_key_pattern = "(?<![A-Z0-9])[A-Z0-9]{20}(?![A-Z0-9])" # nosec
secret_key_pattern = "(?<![A-Za-z0-9/+=])[A-Za-z0-9/+=]{40}(?![A-Za-z0-9/+=])" # nosec
linode_token_pattern = "(?<![A-Za-z0-9/+=])[A-Za-z0-9/+=]{64}(?![A-Za-z0-9/+=])" # nosec
bridgecrew_token_pattern = r"^[a-f0-9]{8}-?[a-f0-9]{4}-?4[a-f0-9]{3}-?[89ab][a-f0-9]{3}-?[a-f0-9]{12}\Z" # nosec
YAML_COMMENT_MARK = '#'



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check
from checkov.kubernetes.checks.Tiller import Tiller


class TillerDeploymentListener(BaseK8Check):

    def __init__(self):
        name = "Ensure the Tiller Deployment (Helm V2) is not accessible from within the cluster"
        id = "CKV_K8S_45"
        # Location: container .image
        supported_kind = ['containers', 'initContainers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return f'{conf["parent"]} - {conf["name"]}' if conf.get('name') else conf["parent"]

    def scan_spec_conf(self, conf):

        is_tiller = Tiller.is_tiller(conf)
        
        if not is_tiller:
            return CheckResult.UNKNOWN

        args = conf.get('args')
        if args:
            for arg in args:
                if '--listen' in arg and ('localhost' in arg or '127.0.0.1' in arg):
                    return CheckResult.PASSED

        return CheckResult.FAILED

check = TillerDeploymentListener()



from typing import Dict, List, Any

from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class SecretManagerSecretEncrypted(BaseResourceCheck):

    def __init__(self):
        name = "Ensure that Secrets Manager secret is encrypted using KMS CMK"
        id = "CKV_AWS_149"
        supported_resources = ["aws_secretsmanager_secret"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        aws_kms_alias = 'aws/'
        kms_key_id = force_list(conf.get('kms_key_id', []))
        if not kms_key_id:
            return CheckResult.FAILED
        else:
            return CheckResult.FAILED if aws_kms_alias in kms_key_id[0] else CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['kms_key_id']


check = SecretManagerSecretEncrypted()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.enums import CheckResult


class RedshiftClusterAllowVersionUpgrade(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensured that redshift cluster allowing version upgrade by default"
        id = "CKV_AWS_141"
        supported_resources = ['aws_redshift_cluster']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return "allow_version_upgrade"


check = RedshiftClusterAllowVersionUpgrade()



from dataclasses import dataclass

from checkov.common.graph.graph_builder.graph_components.block_types import BlockType as CommonBlockType


@dataclass
class BlockType(CommonBlockType):
    METADATA = "metadata"
    PARAMETERS = "parameters"
    RULES = "rules"
    MAPPINGS = "mappings"
    CONDITIONS = "conditions"
    TRANSFORM = "transform"
    OUTPUTS = "outputs"



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import List


class IAMPolicyAttachedToGroupOrRoles(BaseResourceCheck):
    def __init__(self):
        name = "Ensure IAM policies are attached only to groups or roles (Reducing access management complexity may " \
               "in-turn reduce opportunity for a principal to inadvertently receive or retain excessive privileges.)"
        id = "CKV_AWS_40"
        supported_resources = ['aws_iam_user_policy_attachment', 'aws_iam_user_policy', 'aws_iam_policy_attachment']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'user' in conf.keys():
            return CheckResult.FAILED
        if 'users' in conf.keys() and conf['users'][0] is None: #"users": null case 
            return CheckResult.PASSED
        if 'users' in conf.keys() and len(conf['users'][0]) > 0:
            return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['user', 'users']


check = IAMPolicyAttachedToGroupOrRoles()



import copy

from checkov.common.parsers.node import DictNode


def convert_cloudformation_conf_to_iam_policy(conf: DictNode) -> DictNode:
    """
        converts terraform parsed configuration to iam policy document
    """
    result = copy.deepcopy(conf)
    if "Statement" in result.keys():
        result["Statement"] = result.pop("Statement")
        for statement in map(dict, result["Statement"]):
            if "Action" in statement:
                statement["Action"] = str(statement.pop("Action")[0])
            if "Resource" in statement:
                resources = statement.pop("Resource")
                if isinstance(resources, list):
                    statement["Resource"] = str(resources[0])
                else:
                    statement["Resource"] = str(resources)
            if "NotAction" in statement:
                statement["NotAction"] = str(statement.pop("NotAction")[0])
            if "NotResource" in statement:
                not_resources = statement.pop("NotResource")
                if isinstance(not_resources, list):
                    statement["NotResource"] = str(not_resources[0])
                else:
                    statement["NotResource"] = str(not_resources)
            if "Effect" in statement:
                statement["Effect"] = str(statement.pop("Effect"))
            if "Effect" not in statement:
                statement["Effect"] = "Allow"
    return result



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class InstanceMonitoringEnabled(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure OCI Compute Instance has monitoring enabled"
        id = "CKV_OCI_6"
        supported_resources = ['oci_core_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "agent_config/[0]/is_monitoring_disabled"

    def get_forbidden_values(self):
        return [True]

check = InstanceMonitoringEnabled()

